/*
 * Created on Oct 5, 2006
 *
 */
package jPDFViewerSamples.bookmark;

import jPDFViewerSamples.SampleUtil;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import nanoxml.XMLElement;
import nanoxml.XMLParseException;

import com.qoppa.pdf.PDFException;
import com.qoppa.pdfViewer.PDFToolbar;
import com.qoppa.pdfViewer.PDFViewerBean;

public class PDFViewerBeanBookmarks extends PDFViewerBean
{
    private final static String CMD_BOOKMARKMENU = "BookmarksMenu";
    private final static String CMD_GOTOBOOKMARK = "GotoBookmark";
    private final static String CMD_REMOVEBOOKMARK = "RemoveBookmark";
    private final static String CMD_ADD_BOOKMARK = "AddBookmark";
    
    private final static String KEY_REFERENCE = "BMRef";
    private final static String BOOKMARKS_FILENAME = ".jPDFViewerBM.xml";
    private final static File BOOKMARKS_FILE = new File (new File (System.getProperty("user.home")), BOOKMARKS_FILENAME);
    private final static String BOOKMARK_LABEL = "Bookmark";
    
    // XML Keys
    private final static String KEY_NAME = "Name";
    private final static String KEY_SOURCE = "Source";
    private final static String KEY_SOURCETYPE = "SourceType";
    private final static String KEY_PAGENUMBER = "PageNumber";
    
    // List of bookmarks
    private Vector m_Bookmarks;
    
    // Bookmarks menu
    private JButton m_BookmarkButton;
    private JPopupMenu m_BookmarksMenu; 
    
    public PDFViewerBeanBookmarks ()
    {
        // Load bookmarks
        try
        {
            m_Bookmarks = loadBookmarks();
        }
        catch (Throwable t)
        {
            m_Bookmarks = new Vector();
        }
    }
    
    protected PDFToolbar createToolbar()
    {
        PDFToolbar toolbar = super.createToolbar();
        
        // Add the bookmark button
        m_BookmarkButton = new JButton ();
        URL iconURL = getClass().getResource("/jPDFViewerSamples/images/bkmark.gif");
        if (iconURL != null)
        {
            m_BookmarkButton.setIcon(new ImageIcon(iconURL));
        }        
        m_BookmarkButton.setActionCommand(CMD_BOOKMARKMENU);
        m_BookmarkButton.addActionListener(this);
        m_BookmarkButton.setPreferredSize(new java.awt.Dimension((int)(24 * SampleUtil.getDPIScalingMultiplier()),(int)(21 * SampleUtil.getDPIScalingMultiplier())));
        m_BookmarkButton.setMaximumSize(new java.awt.Dimension((int)(24 * SampleUtil.getDPIScalingMultiplier()),(int)(21 * SampleUtil.getDPIScalingMultiplier())));
        m_BookmarkButton.setMinimumSize(new java.awt.Dimension((int)(24 * SampleUtil.getDPIScalingMultiplier()),(int)(21 * SampleUtil.getDPIScalingMultiplier())));

        toolbar.add (m_BookmarkButton);
        
        return toolbar;
    }
    
    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand() == CMD_BOOKMARKMENU)
        {
            getBookmarksMenu().show(m_BookmarkButton, m_BookmarkButton.getWidth() / 2, m_BookmarkButton.getHeight() / 2);
        }
        else if (e.getActionCommand() == CMD_ADD_BOOKMARK)
        {
            cmdAddBookmark();
        }
        else if (e.getActionCommand() == CMD_GOTOBOOKMARK)
        {
            cmdGotoBookmark((BookmarkInfo)((JComponent)e.getSource()).getClientProperty(KEY_REFERENCE));
        }
        else if (e.getActionCommand() == CMD_REMOVEBOOKMARK)
        {
            cmdRemoveBookmark((BookmarkInfo)((JComponent)e.getSource()).getClientProperty(KEY_REFERENCE));
        }
        else
        {
            super.actionPerformed(e);
        }
    }
    
    /**
     * Go to a bookmark.  This method will load the PDF from a file or a URL (according
     * to the type of bookmark) and then set the page in the PDF to that of the bookmark.
     * 
     * @param bi The bookmark information object.
     */
    private void cmdGotoBookmark(final BookmarkInfo bi)
    {
        try
        {
            if (bi.getSourceType() == BookmarkInfo.ST_FILE)
            {
                // Load the PDF document
                loadPDF(bi.getSource());
            }
            else
            {
                loadPDF (new URL (bi.getSource()));
            }
            
            // Set the page.  Need to use invokeLater because the document needs
            // to be layed out before we can call setPage
            try
            {
                SwingUtilities.invokeLater(
                    new Runnable() 
                    {
                        public void run()
                        {
                            setPage(bi.getPageNumber());                
                        };
                    });
            }
            catch (Throwable t)
            {t.printStackTrace();}         
        }
        catch (PDFException pdfE)
        {
            JOptionPane.showMessageDialog(this, pdfE.getMessage());
        }
        catch (MalformedURLException urlE)
        {
            JOptionPane.showMessageDialog(this, "Invalid URL: " + bi.getSource());
        }
    }
    
    /**
     * Remove a bookmarks from the list.  The method will first prompt the user to 
     * confirm the deletion of the bookmark, then remove from the list and save the
     * bookmark file.
     * 
     * @param bi The bookmark information object.
     */
    private void cmdRemoveBookmark (BookmarkInfo bi)
    {
        int rc = JOptionPane.showConfirmDialog(this, "Remove " + bi.getName() + "?", "Remove Bookmark", JOptionPane.OK_CANCEL_OPTION);
        if (rc == JOptionPane.OK_OPTION)
        {
            int bmIndex = m_Bookmarks.indexOf(bi);
            if (bmIndex != -1)
            {
                m_Bookmarks.remove(bmIndex);
                m_BookmarksMenu = null;
                
                try
                {
                    saveBookmarks();
                }
                catch (IOException ioE)
                {
                    m_Bookmarks.insertElementAt(bi, bmIndex);
                    JOptionPane.showMessageDialog (this, "Error saving bookmarks: " + ioE.getMessage());
                }
            }
            else
            {
                JOptionPane.showMessageDialog(this, "Error: Invalid Bookmark");
            }
        }
    }

    /**
     * Returns the menu structure to add, goto and remove bookmarks.  If the menu
     * has not been created, it creates the structure on the fly.
     * 
     * @return The PopupMenu to work with bookmarks.
     */
    private JPopupMenu getBookmarksMenu()
    {
        if (m_BookmarksMenu == null)
        {
            m_BookmarksMenu = new JPopupMenu ();
            
            // Goto bookmarks
            JMenu subMenu = new JMenu ("Go to Bookmark");
            addBookmarksToMenu (subMenu, CMD_GOTOBOOKMARK);
            m_BookmarksMenu.add (subMenu);
            m_BookmarksMenu.add (new JPopupMenu.Separator ());
            
            // Set bookmarks
            JMenuItem item = new JMenuItem ("Add Bookmark...");
            item.setActionCommand(CMD_ADD_BOOKMARK);
            item.addActionListener(this);
            m_BookmarksMenu.add(item);

            // Remove bookmarks
            subMenu = new JMenu ("Remove Bookmark");
            addBookmarksToMenu (subMenu, CMD_REMOVEBOOKMARK);
            m_BookmarksMenu.add (subMenu);
        }
        return m_BookmarksMenu;
    }
    
    private void addBookmarksToMenu (JMenu parentMenu, String actionCommand)
    {
        if (m_Bookmarks == null || m_Bookmarks.size() == 0)
        {
            JMenuItem item = new JMenuItem ("<No Bookmarks>");
            item.setEnabled(false);
            parentMenu.add (item);
        }
        else
        {
            for (int count = 0; count < m_Bookmarks.size(); ++count)
            {
                BookmarkInfo bi = (BookmarkInfo)m_Bookmarks.get (count);
                JMenuItem item = new JMenuItem (bi.getName());
                item.setActionCommand(actionCommand);
                item.addActionListener(this);
                item.putClientProperty(KEY_REFERENCE, bi);
                parentMenu.add (item);
            }
        }
    }
    
    /**
     * Saves the bookmars to an XML file in the user's home directory.
     * 
     * @throws IOException
     */
    private void saveBookmarks() throws IOException
    {
        // Create the XML tree
        XMLElement bkRoot = new XMLElement ("Root");
        for (int count = 0; count < m_Bookmarks.size(); ++count)
        {
            BookmarkInfo bi = (BookmarkInfo)m_Bookmarks.get (count);
            XMLElement xmlBM = new XMLElement (BOOKMARK_LABEL);
            xmlBM.setAttribute (KEY_NAME, bi.getName());
            xmlBM.setAttribute (KEY_SOURCE, bi.getSource());
            xmlBM.setIntAttribute (KEY_SOURCETYPE, bi.getSourceType());
            xmlBM.setIntAttribute (KEY_PAGENUMBER, bi.getPageNumber());
            
            bkRoot.addChild(xmlBM);
        }
        
        // Save XML settings file
        bkRoot.write(BOOKMARKS_FILE);
    }
    
    private Vector loadBookmarks () throws IOException, PDFException
    {
        // Create bookmark list
        Vector bookmarks = new Vector();
        
        // Make sure that there is a bookmarks file
        if (BOOKMARKS_FILE.exists() == false)
        {
            return bookmarks;
        }
        
        // Read the XML file containing bookmarks
        try
        {
            // parse the XML file
            XMLElement bkRoot = new XMLElement ();
            bkRoot.parseFromFile(BOOKMARKS_FILE);

            // Loop through the XML file
            int childCount = bkRoot.getMutableChildren().size();
            for (int count = 0; count < childCount; ++count)
            {
                XMLElement child = (XMLElement)bkRoot.getMutableChildren().get (count);
                if (BOOKMARK_LABEL.equals (child.getName()))
                {
                    // Add new bookmark
                    BookmarkInfo bki = new BookmarkInfo (child.getStringAttribute(KEY_NAME),
                                                            child.getStringAttribute(KEY_SOURCE),
                                                            child.getIntAttribute(KEY_SOURCETYPE, BookmarkInfo.ST_FILE), 
                                                            child.getIntAttribute(KEY_PAGENUMBER, 1));
                    bookmarks.add (bki);
                }
            }
            return bookmarks;
        }
        catch (XMLParseException pe)
        {
            try
            {
                BOOKMARKS_FILE.delete();
            }
            catch (Throwable t) {}
            
            throw new PDFException ("Error reading bookmarks.");
        }
    }
    
    /**
     * Add a bookmark.  This method will display the new bookmark dialog to let the user
     * enter a name for the new bookmark and to confirm the save.
     *
     */
    private void cmdAddBookmark()
    {
        // Create bookmark
        BookmarkInfo bk = null;
        String bkName = getBkName();
        if (getDocument() != null && getDocument().getPDFSource() != null)
        {
            bk = new BookmarkInfo (bkName, getDocument().getPDFSource().getPath(), BookmarkInfo.ST_FILE, getPageNumber());
        }
        else
        {
            JOptionPane.showMessageDialog(this, "Unable to create bookmark for this document.");
        }

        // Show the bookmark dialog
        AddBookmarkController abc = new AddBookmarkController();
        bk = abc.showDialog ((Frame)SwingUtilities.windowForComponent(this), bk);
        if (bk != null)
        {
            m_BookmarksMenu = null;
            m_Bookmarks.add (bk);
            
            try
            {
                saveBookmarks();
            }
            catch (IOException ioE)
            {
                m_Bookmarks.remove(m_Bookmarks.size()-1);
                JOptionPane.showMessageDialog (this, "Error saving bookmarks: " + ioE.getMessage());
            }
        }
    }
    
    private String getBkName()
    {
        HashSet nameSet = new HashSet ();
        for (int count = 0; count < m_Bookmarks.size(); ++count)
        {
            nameSet.add (((BookmarkInfo)m_Bookmarks.get (count)).getName());
        }
        
        int count = 1;
        do
        {
            String newName = "Bookmark" + count;
            if (nameSet.contains(newName) == false)
            {
                return newName;
            }
            ++count;
        } while (true);
    }
}
