package jPDFViewerSamples.history;

import jPDFViewerSamples.SampleUtil;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import com.qoppa.pdf.PDFException;
import com.qoppa.pdfViewer.PDFViewerBean;
import com.qoppa.pdfViewer.history.HistoryListener;
import com.qoppa.pdfViewer.history.LocationHistory;

/**
 * This sample demonstrates how to implement a HistoryListener to maintain a
 * document browsing history for all documents that are loaded. When a document
 * is loaded, PDFViewerBean creates and updates a LocationHistory for each
 * document. The document history list used to load previously loaded documents
 * in nextDocument() and previousDocument(). 
 * 
 */
public class MultipleDocumentHistory implements HistoryListener
{
	private PDFViewerBean m_ViewerBean;
	private JButton m_NextViewButton;
	private JButton m_PrevViewButton;

	/**
	 * List of previously visited documents and their corresponding LocationHistory
	 */
	private List<DocumentHistory> m_DocHistory;

	/**
	 * Index in the document history list of the currently displayed document
	 */
	private int m_DocHistoryIndex;

	/**
	 * This flag is used to ignore the events generated (locationChanged(),
	 * beforeDocumentSet(), afterDocumentSet()) when loading a document from the
	 * m_DocHistory list. Those events only need to be processed when loading a
	 * new document.
	 */
	private boolean m_IgnoreEvents;

	public MultipleDocumentHistory()
	{
		init();
	}

	private void init()
	{
		m_DocHistory = new ArrayList<DocumentHistory>();
		m_DocHistoryIndex = -1;

		m_ViewerBean = new PDFViewerBean();
		m_ViewerBean.getToolbar().add(getPrevViewButton());
		m_ViewerBean.getToolbar().add(getNextViewButton());
		m_ViewerBean.addHistoryListener(this);

		getPrevViewButton().addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e)
			{
				m_ViewerBean.gotoPreviousView();
			}
		});

		getNextViewButton().addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e)
			{
				m_ViewerBean.gotoNextView();
			}
		});

		toggleButtons(m_ViewerBean);
	}

	/**
	 * Displays the PDFViewerBean in a new JFrame.
	 */
	public void showViewerBean()
	{
		JFrame frame = new JFrame();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setContentPane(m_ViewerBean);
		frame.setSize((int)(1000 * SampleUtil.getDPIScalingMultiplier()), (int)(600 * SampleUtil.getDPIScalingMultiplier()));
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}

	private JButton getPrevViewButton()
	{
		if (m_PrevViewButton == null)
		{
			m_PrevViewButton = new JButton("<< Prev");
		}
		return m_PrevViewButton;
	}

	private JButton getNextViewButton()
	{
		if (m_NextViewButton == null)
		{
			m_NextViewButton = new JButton("Next >>");
		}
		return m_NextViewButton;
	}

	public void previousDocument(PDFViewerBean viewerBean)
	{
		// if there is a "previous" document in m_DocHistory, load the
		// document and restore its LocationHistory
		if (m_DocHistoryIndex > 0)
		{
			m_DocHistoryIndex--;
			DocumentHistory history = m_DocHistory.get(m_DocHistoryIndex);
			m_IgnoreEvents = true;
			try
			{
				viewerBean.loadPDF(history.getFilePath());

				// Set the LocationHistory on the PDFViewerBean
				viewerBean.setLocationHistory(history.getDocumentViewHistory());

				// Update the Next/Prev buttons
				toggleButtons(viewerBean);
			}
			catch (PDFException e)
			{
				e.printStackTrace();
			}
			finally
			{
				m_IgnoreEvents = false;
			}
		}
	}

	public void nextDocument(PDFViewerBean viewerBean)
	{
		// If there is a "next" document in m_DocHistory, load the
		// document and restore its LocationHistory
		if (m_DocHistoryIndex < m_DocHistory.size() - 1)
		{
			m_DocHistoryIndex++;
			DocumentHistory history = m_DocHistory.get(m_DocHistoryIndex);
			m_IgnoreEvents = true;
			try
			{
				viewerBean.loadPDF(history.getFilePath());

				// Set the LocationHistory on the PDFViewerBean
				viewerBean.setLocationHistory(history.getDocumentViewHistory());

				// Update the Next/Prev buttons
				toggleButtons(viewerBean);
			}
			catch (PDFException e)
			{
				e.printStackTrace();
			}
			finally
			{
				m_IgnoreEvents = false;
			}
		}
	}

	public void beforeDocumentSet(PDFViewerBean viewerBean)
	{
		if (!m_IgnoreEvents)
		{
			// if there is an existing history object, clear the "next" history
			if (viewerBean.getDocument() != null)
			{
				viewerBean.getLocationHistory().clearNext();
			}

			// if we're not at the end of the doc history list, clear any "next" documents
			clearNextDocuments();
		}
	}

	public void afterDocumentSet(PDFViewerBean viewerBean)
	{
		// if there is no document, clear the document history
		if (viewerBean.getDocument() == null)
		{
			m_DocHistory.clear();
			m_DocHistoryIndex = -1;
		}
		else if (!m_IgnoreEvents)
		{
			// add the newly opened document's LocationHistory to m_DocHistory
			m_DocHistory.add(new DocumentHistory(viewerBean.getDocument().getPDFSource().getPath(), viewerBean.getLocationHistory()));
			m_DocHistoryIndex++;
		}

		// since history may have been added or the document may have been cleared, toggle the buttons
		toggleButtons(viewerBean);
	}

	public void locationChanged(PDFViewerBean viewerBean, boolean newLocation)
	{
		if (!m_IgnoreEvents && newLocation)
		{
			// if we're not at the end of the doc history list, clear any "next" documents
			clearNextDocuments();
		}
		toggleButtons(viewerBean);
	}

	/**
	 * Clear the "next" documents
	 */
	private void clearNextDocuments()
	{
		if (m_DocHistoryIndex + 1 < m_DocHistory.size())
		{
			m_DocHistory = m_DocHistory.subList(0, m_DocHistoryIndex + 1);
		}
	}

	// Toggle the next and previous history buttons
	private void toggleButtons(PDFViewerBean viewerBean)
	{
		if (viewerBean.getDocument() == null)
		{
			getNextViewButton().setEnabled(false);
			getPrevViewButton().setEnabled(false);
		}
		else
		{
			boolean hasNext = viewerBean.getLocationHistory().hasNext() || m_DocHistory.size() > m_DocHistoryIndex + 1;
			getNextViewButton().setEnabled(hasNext);
			boolean hasPrevious = viewerBean.getLocationHistory().hasPrevious() || m_DocHistoryIndex > 0;
			getPrevViewButton().setEnabled(hasPrevious);
		}
	}

	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run()
			{
				MultipleDocumentHistory sample = new MultipleDocumentHistory();
				sample.showViewerBean();
			}
		});
	}

	/**
	 * Utility class to contain the document path with it's corresponding
	 * <code>LocationHistory</code>. This sample only uses
	 * <code>FilePDFSource</code>'s, so the file path is stored with it's
	 * <code>LocationHistory</code>. It is not recommended to hold a reference
	 * to <code>IPDFDocument</code> or <code>PDFSource</code> as this would
	 * cause every document loaded to be held in memory. It is preferable to
	 * only keep an identifier for the document (ie. document path, database
	 * record id, ...)
	 */
	private class DocumentHistory
	{
		private String m_FilePath;
		private LocationHistory m_LocationHistory;

		private DocumentHistory(String path, LocationHistory history)
		{
			m_FilePath = path;
			m_LocationHistory = history;
		}

		private String getFilePath()
		{
			return m_FilePath;
		}

		private LocationHistory getDocumentViewHistory()
		{
			return m_LocationHistory;
		}
	}
}
